@@ -1,4 +1,4 @@
-package Zonemaster::Constants v1.0.0;
+package Zonemaster::Constants v1.1.0;
 
 use strict;
 use warnings;
@@ -27,6 +27,7 @@ our @EXPORT_OK = qw[
   $SOA_REFRESH_MINIMUM_VALUE
   $SOA_RETRY_MINIMUM_VALUE
   $UDP_PAYLOAD_LIMIT
+  $UDP_COMMON_EDNS_LIMIT
   @IPV4_SPECIAL_ADDRESSES
   @IPV6_SPECIAL_ADDRESSES
 ];
@@ -41,7 +42,7 @@ our %EXPORT_TAGS = (
     soa  => [
         qw($SOA_DEFAULT_TTL_MAXIMUM_VALUE $SOA_DEFAULT_TTL_MINIMUM_VALUE $SOA_EXPIRE_MINIMUM_VALUE $SOA_REFRESH_MINIMUM_VALUE $SOA_RETRY_MINIMUM_VALUE $DURATION_12_HOURS_IN_SECONDS $DURATION_180_DAYS_IN_SECONDS $MAX_SERIAL_VARIATION)
     ],
-    misc      => [qw($UDP_PAYLOAD_LIMIT $MINIMUM_NUMBER_OF_NAMESERVERS)],
+    misc      => [qw($UDP_PAYLOAD_LIMIT $UDP_COMMON_EDNS_LIMIT $MINIMUM_NUMBER_OF_NAMESERVERS)],
     addresses => [qw(@IPV4_SPECIAL_ADDRESSES @IPV6_SPECIAL_ADDRESSES)],
 );
 
@@ -71,7 +72,8 @@ Readonly our $SOA_EXPIRE_MINIMUM_VALUE      => 604_800;    # 1 week
 Readonly our $SOA_REFRESH_MINIMUM_VALUE     => 14_400;     # 4 hours
 Readonly our $SOA_RETRY_MINIMUM_VALUE       => 3_600;      # 1 hour
 
-Readonly our $UDP_PAYLOAD_LIMIT => 512;
+Readonly our $UDP_PAYLOAD_LIMIT     => 512;
+Readonly our $UDP_COMMON_EDNS_LIMIT => 4_096;
 
 Readonly::Array our @IPV4_SPECIAL_ADDRESSES => (
     { ip => Net::IP::XS->new( q{0.0.0.0/8} ),          name => q{This host on this network}, reference => q{RFC 1122} },
@@ -137,10 +139,6 @@ All exportable names.
 
 DNSSEC algorithms.
 
-=item asn
-
-Constants used by the ASN test module.
-
 =item name
 
 Label and name lengths.
@@ -189,38 +187,6 @@ C<$ALGO_STATUS_VALID>
 
 =item *
 
-C<$ASN_CHECKING_ROUTE_VIEWS_SERVICE_NAME>
-
-=item *
-
-C<$ASN_CHECKING_SERVICE_USED>
-
-=item *
-
-C<$ASN_CHECKING_ZONEMASTER_SERVICE_NAME>
-
-=item *
-
-C<$ASN_IPV4_CHECKING_SERVICE_ROUTE_VIEWS_DOMAIN>
-
-=item *
-
-C<$ASN_IPV4_CHECKING_SERVICE_ZONEMASTER_DOMAIN>
-
-=item *
-
-C<$ASN_IPV6_CHECKING_SERVICE_ROUTE_VIEWS_DOMAIN>
-
-=item *
-
-C<$ASN_IPV6_CHECKING_SERVICE_ZONEMASTER_DOMAIN>
-
-=item *
-
-C<$ASN_UNASSIGNED_UNANNOUNCED_ADDRESS_SPACE_VALUE>
-
-=item *
-
 C<$DURATION_12_HOURS_IN_SECONDS>
 
 =item *
@@ -277,6 +243,10 @@ C<$UDP_PAYLOAD_LIMIT>
 
 =item *
 
+C<UDP_COMMON_EDNS_LIMIT>
+
+=item *
+
 C<@IPV4_SPECIAL_ADDRESSES>
 
 =item *
@@ -284,3 +254,5 @@ C<@IPV4_SPECIAL_ADDRESSES>
 C<@IPV6_SPECIAL_ADDRESSES>
 
 =back
+
+=cut
