@@ -1,4 +1,4 @@
-package Zonemaster::Test::Connectivity v1.0.2;
+package Zonemaster::Test::Connectivity v1.0.5;
 
 use strict;
 use warnings;
@@ -72,6 +72,11 @@ sub metadata {
               NAMESERVERS_WITH_UNIQ_AS
               IPV4_ASN
               IPV6_ASN
+              ASN_INFOS_RAW
+              ASN_INFOS_ANNOUNCE_BY
+              ASN_INFOS_ANNOUNCE_IN
+              IPV4_DISABLED
+              IPV6_DISABLED
               )
         ],
     };
@@ -92,10 +97,13 @@ sub translation {
         'NAMESERVER_HAS_UDP_53'             => 'Nameserver {ns}/{address} accessible over UDP on port 53.',
         'NAMESERVER_NO_TCP_53'              => 'Nameserver {ns}/{address} not accessible over TCP on port 53.',
         'NAMESERVER_NO_UDP_53'              => 'Nameserver {ns}/{address} not accessible over UDP on port 53.',
-        'IPV4_DISABLED'                     => 'IPv4 is disabled, not sending "{type}" query to {ns}.',
-        'IPV6_DISABLED'                     => 'IPv6 is disabled, not sending "{type}" query to {ns}.',
+        'IPV4_DISABLED'                     => 'IPv4 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
+        'IPV6_DISABLED'                     => 'IPv6 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
         'IPV4_ASN'                          => 'Name servers have IPv4 addresses in the following ASs: {asn}.',
         'IPV6_ASN'                          => 'Name servers have IPv6 addresses in the following ASs: {asn}.',
+        'ASN_INFOS_RAW'                     => '[ASN:RAW] {address};{data}',
+        'ASN_INFOS_ANNOUNCE_BY'             => '[ASN:ANNOUNCE_BY] {address};{asn}',
+        'ASN_INFOS_ANNOUNCE_IN'             => '[ASN:ANNOUNCE_IN] {address};{prefix}',
     };
 } ## end sub translation
 
@@ -110,6 +118,7 @@ sub version {
 sub connectivity01 {
     my ( $class, $zone ) = @_;
     my @results;
+    my $query_type = q{SOA};
 
     my %ips;
 
@@ -121,8 +130,9 @@ sub connectivity01 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns   => "$local_ns",
-                    type => q{SOA},
+                    ns      => $local_ns->name->string,
+                    address => $local_ns->address->short,
+                    rrtype  => $query_type,
                 }
               );
             next;
@@ -132,8 +142,9 @@ sub connectivity01 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns   => "$local_ns",
-                    type => q{SOA},
+                    ns      => $local_ns->name->string,
+                    address => $local_ns->address->short,
+                    rrtype  => $query_type,
                 }
               );
             next;
@@ -141,7 +152,7 @@ sub connectivity01 {
 
         next if $ips{ $local_ns->address->short };
 
-        my $p = $local_ns->query( $zone->name, q{SOA}, { usevc => 0 } );
+        my $p = $local_ns->query( $zone->name, $query_type, { usevc => 0 } );
 
         if ( $p and $p->rcode eq q{NOERROR} ) {
             push @results,
@@ -173,6 +184,7 @@ sub connectivity02 {
     my ( $class, $zone ) = @_;
     my @results;
     my %ips;
+    my $query_type = q{SOA};
 
     foreach
       my $local_ns ( @{ Zonemaster::TestMethods->method4( $zone ) }, @{ Zonemaster::TestMethods->method5( $zone ) } )
@@ -182,8 +194,9 @@ sub connectivity02 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns   => "$local_ns",
-                    type => q{SOA},
+                    ns      => $local_ns->name->string,
+                    address => $local_ns->address->short,
+                    rrtype  => $query_type,
                 }
               );
             next;
@@ -193,8 +206,9 @@ sub connectivity02 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns   => "$local_ns",
-                    type => q{SOA},
+                    ns      => $local_ns->name->string,
+                    address => $local_ns->address->short,
+                    rrtype  => $query_type,
                 }
               );
             next;
@@ -202,7 +216,7 @@ sub connectivity02 {
 
         next if $ips{ $local_ns->address->short };
 
-        my $p = $local_ns->query( $zone->name, q{SOA}, { usevc => 1 } );
+        my $p = $local_ns->query( $zone->name, $query_type, { usevc => 1 } );
 
         if ( $p and $p->rcode eq q{NOERROR} ) {
             push @results,
@@ -234,18 +248,86 @@ sub connectivity03 {
     my ( $class, $zone ) = @_;
     my @results;
 
-    my %ips = ( 4 => {}, 6 => {} );
+    my %ips = ( $IP_VERSION_4 => {}, $IP_VERSION_6 => {} );
 
-    foreach my $ns ( @{ Zonemaster::TestMethods->method4( $zone ) }, @{ Zonemaster::TestMethods->method5( $zone ) } ) {
+    foreach my $ns ( @{ Zonemaster::TestMethods->method4( $zone ) } ) {
         my $addr = $ns->address;
         $ips{ $addr->version }{ $addr->ip } = $addr;
     }
 
-    my @v4ips = values %{ $ips{4} };
-    my @v6ips = values %{ $ips{6} };
+    my @v4ips = values %{ $ips{$IP_VERSION_4} };
+    my @v6ips = values %{ $ips{$IP_VERSION_6} };
 
-    my @v4asns = uniq grep { $_ } map { Zonemaster::ASNLookup->get( $_ ) } @v4ips;
-    my @v6asns = uniq grep { $_ } map { Zonemaster::ASNLookup->get( $_ ) } @v6ips;
+    my @v4asns;
+    my @v6asns;
+
+    foreach my $v4ip ( @v4ips ) {
+        my ( $asnref, $prefix, $raw ) = Zonemaster::ASNLookup->get_with_prefix( $v4ip );
+        if ( $raw ) {
+            push @results,
+              info(
+                ASN_INFOS_RAW => {
+                    address => $v4ip->short,
+                    data    => $raw,
+                }
+              );
+        }
+        if ( $asnref ) {
+            push @results,
+              info(
+                ASN_INFOS_ANNOUNCE_BY => {
+                    address => $v4ip->short,
+                    asn     => join( q{,}, @{$asnref} ),
+                }
+              );
+            push @v4asns, @{$asnref};
+        }
+        if ( $prefix ) {
+            push @results,
+              info(
+                ASN_INFOS_ANNOUNCE_IN => {
+                    address => $v4ip->short,
+                    prefix  => sprintf "%s/%d",
+                    $prefix->ip, $prefix->prefixlen,
+                }
+              );
+        }
+    } ## end foreach my $v4ip ( @v4ips )
+    foreach my $v6ip ( @v6ips ) {
+        my ( $asnref, $prefix, $raw ) = Zonemaster::ASNLookup->get_with_prefix( $v6ip );
+        if ( $raw ) {
+            push @results,
+              info(
+                ASN_INFOS_RAW => {
+                    address => $v6ip->short,
+                    data    => $raw,
+                }
+              );
+        }
+        if ( $asnref ) {
+            push @results,
+              info(
+                ASN_INFOS_ANNOUNCE_BY => {
+                    address => $v6ip->short,
+                    asn     => join( q{,}, @{$asnref} ),
+                }
+              );
+            push @v6asns, @{$asnref};
+        }
+        if ( $prefix ) {
+            push @results,
+              info(
+                ASN_INFOS_ANNOUNCE_IN => {
+                    address => $v6ip->short,
+                    prefix  => sprintf "%s/%d",
+                    $prefix->short, $prefix->prefixlen,
+                }
+              );
+        }
+    } ## end foreach my $v6ip ( @v6ips )
+
+    @v4asns = uniq @v4asns;
+    @v6asns = uniq @v6asns;
     my @all_asns = uniq( @v4asns, @v6asns );
 
     push @results, info( IPV4_ASN => { asn => \@v4asns } );
