@@ -1,4 +1,4 @@
-package Zonemaster::Test::Delegation v1.0.3;
+package Zonemaster::Test::Delegation v1.0.4;
 
 use strict;
 use warnings;
@@ -103,23 +103,23 @@ sub translation {
         "EXTRA_NAME_CHILD" => "Child has nameserver(s) not listed at parent ({extra}).",
         "REFERRAL_SIZE_OK" => "The smallest possible legal referral packet is smaller than 513 octets (it is {size}).",
         "IS_NOT_AUTHORITATIVE" => "Nameserver {ns} response is not authoritative on {proto} port 53.",
-        "ENOUGH_NS_GLUE"       => "Parent lists enough nameservers ({glue}). Lower limit set to {minimum}.",
+        "ENOUGH_NS_GLUE"       => "Parent lists enough ({count}) nameservers ({glue}). Lower limit set to {minimum}.",
         "NS_RR_IS_CNAME"       => "Nameserver {ns} {address_type} RR point to CNAME.",
         "SAME_IP_ADDRESS"      => "IP {address} refers to multiple nameservers ({nss}).",
         "DISTINCT_IP_ADDRESS"  => "All the IP addresses used by the nameservers are unique",
-        "ENOUGH_NS"            => "Child lists enough nameservers ({ns}). Lower limit set to {minimum}.",
+        "ENOUGH_NS"            => "Child lists enough ({count}) nameservers ({ns}). Lower limit set to {minimum}.",
         "NAMES_MATCH"          => "All of the nameserver names are listed both at parent and child.",
         "TOTAL_NAME_MISMATCH"  => "None of the nameservers listed at the parent are listed at the child.",
         "SOA_NOT_EXISTS"       => "A SOA query NOERROR response from {ns} was received empty.",
         "EXTRA_NAME_PARENT"    => "Parent has nameserver(s) not listed at the child ({extra}).",
-        "NOT_ENOUGH_NS_GLUE"   => "Parent does not list enough nameservers ({glue}). Lower limit set to {minimum}.",
-        "NOT_ENOUGH_NS"        => "Child does not list enough nameservers ({ns}). Lower limit set to {minimum}.",
+        "NOT_ENOUGH_NS_GLUE"   => "Parent does not list enough ({count}) nameservers ({glue}). Lower limit set to {minimum}.",
+        "NOT_ENOUGH_NS"        => "Child does not list enough ({count}) nameservers ({ns}). Lower limit set to {minimum}.",
         "ARE_AUTHORITATIVE"    => "All these nameservers are confirmed to be authoritative : {nsset}.",
         "NS_RR_NO_CNAME"       => "No nameserver point to CNAME alias.",
         "SOA_EXISTS"           => "All the nameservers have SOA record.",
-        "ENOUGH_NS_TOTAL"      => "Parent and child list enough nameservers ({ns}). Lower limit set to {minimum}.",
+        "ENOUGH_NS_TOTAL"      => "Parent and child list enough ({count}) nameservers ({ns}). Lower limit set to {minimum}.",
         "NOT_ENOUGH_NS_TOTAL" =>
-          "Parent and child do not list enough nameservers ({ns}). Lower limit set to {minimum}.",
+          "Parent and child do not list enough ({count}) nameservers ({ns}). Lower limit set to {minimum}.",
         'IPV4_DISABLED' => 'IPv4 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
         'IPV6_DISABLED' => 'IPv6 is disabled, not sending "{rrtype}" query to {ns}/{address}.',
     };
@@ -145,7 +145,7 @@ sub delegation01 {
             ENOUGH_NS_GLUE => {
                 count   => scalar( @parent_nsnames ),
                 minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-                glue    => join( q{;}, @parent_nsnames ),
+                glue    => join( q{;}, sort @parent_nsnames ),
             }
           );
     }
@@ -155,7 +155,7 @@ sub delegation01 {
             NOT_ENOUGH_NS_GLUE => {
                 count   => scalar( @parent_nsnames ),
                 minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-                glue    => join( q{;}, @parent_nsnames ),
+                glue    => join( q{;}, sort @parent_nsnames ),
             }
           );
     }
@@ -168,7 +168,7 @@ sub delegation01 {
             ENOUGH_NS => {
                 count   => scalar( @child_nsnames ),
                 minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-                ns      => join( q{;}, @child_nsnames ),
+                ns      => join( q{;}, sort @child_nsnames ),
             }
           );
     }
@@ -178,7 +178,7 @@ sub delegation01 {
             NOT_ENOUGH_NS => {
                 count   => scalar( @child_nsnames ),
                 minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-                ns      => join( q{;}, @child_nsnames ),
+                ns      => join( q{;}, sort @child_nsnames ),
             }
           );
     }
@@ -192,7 +192,7 @@ sub delegation01 {
             ENOUGH_NS_TOTAL => {
                 count   => scalar( @all_nsnames ),
                 minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-                ns      => join( q{;}, @all_nsnames ),
+                ns      => join( q{;}, sort @all_nsnames ),
             }
           );
     }
@@ -202,7 +202,7 @@ sub delegation01 {
             NOT_ENOUGH_NS_TOTAL => {
                 count   => scalar( @all_nsnames ),
                 minimum => $MINIMUM_NUMBER_OF_NAMESERVERS,
-                ns      => join( q{;}, @all_nsnames ),
+                ns      => join( q{;}, sort @all_nsnames ),
             }
           );
     }
@@ -260,7 +260,7 @@ sub delegation03 {
       my $local_ns ( @{ Zonemaster::TestMethods->method4( $zone ) }, @{ Zonemaster::TestMethods->method5( $zone ) } )
     {
         next if $nsnames_and_ip{ $local_ns->name->string . q{/} . $local_ns->address->short };
-        if ( $zone->is_in_zone( $local_ns->name ) ) {
+        if ( $zone->is_in_zone( $local_ns->name->string ) ) {
             push @needs_glue, $local_ns;
         }
         $nsnames_and_ip{ $local_ns->name->string . q{/} . $local_ns->address->short }++;
@@ -327,7 +327,7 @@ sub delegation04 {
                 IPV6_DISABLED => {
                     ns      => $local_ns->name->string,
                     address => $local_ns->address->short,
-                    type    => $query_type,
+                    rrtype  => $query_type,
                 }
               );
             next;
@@ -339,13 +339,13 @@ sub delegation04 {
                 IPV4_DISABLED => {
                     ns      => $local_ns->name->string,
                     address => $local_ns->address->short,
-                    type    => $query_type,
+                    rrtype  => $query_type,
                 }
               );
             next;
         }
 
-        next if $nsnames{ $local_ns->name };
+        next if $nsnames{ $local_ns->name->string };
 
         foreach my $usevc ( 0, 1 ) {
             my $p = $local_ns->query( $zone->name, $query_type, { usevc => $usevc } );
@@ -354,13 +354,13 @@ sub delegation04 {
                     push @results,
                       info(
                         IS_NOT_AUTHORITATIVE => {
-                            ns    => $local_ns->name,
+                            ns    => $local_ns->name->string,
                             proto => $usevc ? q{TCP} : q{UDP},
                         }
                       );
                 }
                 else {
-                    push @authoritatives, $local_ns->name;
+                    push @authoritatives, $local_ns->name->string;
                 }
             }
         }
@@ -380,7 +380,7 @@ sub delegation04 {
         push @results,
           info(
             ARE_AUTHORITATIVE => {
-                nsset => join( q{,}, @authoritatives ),
+                nsset => join( q{,}, uniq sort @authoritatives ),
             }
           );
     }
@@ -444,7 +444,7 @@ sub delegation06 {
                 IPV6_DISABLED => {
                     ns      => $local_ns->name->string,
                     address => $local_ns->address->short,
-                    type    => $query_type,
+                    rrtype  => $query_type,
                 }
               );
             next;
@@ -456,13 +456,13 @@ sub delegation06 {
                 IPV4_DISABLED => {
                     ns      => $local_ns->name->string,
                     address => $local_ns->address->short,
-                    type    => $query_type,
+                    rrtype  => $query_type,
                 }
               );
             next;
         }
 
-        next if $nsnames{ $local_ns->name };
+        next if $nsnames{ $local_ns->name->string };
 
         my $p = $local_ns->query( $zone->name, $query_type );
         if ( $p and $p->rcode eq q{NOERROR} ) {
@@ -470,13 +470,13 @@ sub delegation06 {
                 push @results,
                   info(
                     SOA_NOT_EXISTS => {
-                        ns => $local_ns->name,
+                        ns => $local_ns->name->string,
                     }
                   );
             }
         }
 
-        $nsnames{ $local_ns->name }++;
+        $nsnames{ $local_ns->name->string }++;
     } ## end foreach my $local_ns ( @{ Zonemaster::TestMethods...})
 
     if (
@@ -513,7 +513,7 @@ sub delegation07 {
         push @results,
           info(
             EXTRA_NAME_PARENT => {
-                extra => join( q{;}, @extra_name_parent ),
+                extra => join( q{;}, sort @extra_name_parent ),
             }
           );
     }
@@ -522,7 +522,7 @@ sub delegation07 {
         push @results,
           info(
             EXTRA_NAME_CHILD => {
-                extra => join( q{;}, @extra_name_child ),
+                extra => join( q{;}, sort @extra_name_child ),
             }
           );
     }
@@ -531,7 +531,7 @@ sub delegation07 {
         push @results,
           info(
             NAMES_MATCH => {
-                names => join( q{;}, @same_name ),
+                names => join( q{;}, sort @same_name ),
             }
           );
     }
@@ -540,8 +540,8 @@ sub delegation07 {
         push @results,
           info(
             TOTAL_NAME_MISMATCH => {
-                glue  => join( q{;}, @extra_name_parent ),
-                child => join( q{;}, @extra_name_child ),
+                glue  => join( q{;}, sort @extra_name_parent ),
+                child => join( q{;}, sort @extra_name_child ),
             }
           );
     }
@@ -634,6 +634,7 @@ Verify existence of SOA.
 =item delegation07($zone)
 
 Verify that parent glue name records are present in child.
+
 =back
 
 =cut
